# -*- coding: utf-8 -*-

import simplejson as json
import pprint
from InterUnion import Utils
import math
import operator
import networkx as nx

class nPartiteGraph:

	def __init__(self):
		self.dummy = "bleh"
		self.dbname = "CNRS.sqlite"

	def SameKindNodes( self, Nodes_str2int , OutputGraph , Type , Shape, Nodes , Links , ExtraAtts ):
		# print( NodesxColor )
		ExtrAttsFlag = False
		if ExtraAtts!=False: ExtrAttsFlag=True

		for n in OutputGraph.nodes_iter():
			node = {
				"attributes":{},
				"r_id": n,
				"id": n,
				"label": str(n),
				"type": Type,
				"size": OutputGraph.degree(n)
			}
			if Shape: 
				node["shape"] = Shape
			if ExtrAttsFlag and n in ExtraAtts:
				for key in ExtraAtts[n]:
					if key=="color":
						node["color"] = ExtraAtts[n][key]
					else:
						node["attributes"][key] = ExtraAtts[n][key]
			else:
				node["attributes"]["clust_default"] = 1

			Nodes.append(node)



		c = len( Links )+1
		for e in OutputGraph.edges_iter():
			s = e[0]
			t = e[1]
			link = {
				"id": c,
				"s":s,
				"t":t,
				"w": OutputGraph[s][t]["weight"]
			}
			Links.append(link)
			c += 1
		return Nodes,Links

	#
	# for a in AxB*:
	#	Keys = IDF ( AxB*[a] ,  BxA* , N_docs , threshold )
	#	addCompleteSubGraph ( Keys ) #filling the B-graph.
	#
	#  = = = = = = = = = = = = = = = = = = = = = = = =
	# so From AxB* i'm building my B-graph, where :
	#  = = = = = = = = = = = = = = = = = = = = = = = =
	#	OpposNeigh_i <-> AxB*[a]	: b-neighborhood of node-a
	#	SameNeigh <-> BxA* 			: A-neighborhood of Nodes-B , for getting the number of Nodes-A/opposites of each b in b-neighborhood
	#	N_docs						: number of Nodes-A (like nb of Documents in tf-idf)
	#	threshold					: Max neighborhood size
	def IDF (self , OpposNeigh_i , SameNeigh , N_docs , threshold):
		TFIDFs = {}
		for d in OpposNeigh_i:
			GlobalCount_term = len(SameNeigh[d].keys() )
			tf = 1
			idf = math.log( N_docs/float(GlobalCount_term) )
			TFIDFs[d] = tf*idf

		sorted_x = sorted( TFIDFs.items() , key = operator.itemgetter(1) , reverse=True )
		Keys = []
		for t in sorted_x[:threshold]:
			Keys.append( t[0] )
		# print ID , ":" , len(Keys)  
		return Keys


	def OCC (self , OpposNeigh_i , SameNeigh , N_docs , threshold):
		OCCs = {}
		for d in OpposNeigh_i:
			OCCs[d] = len(SameNeigh[d].keys() )

		sorted_x = sorted( OCCs.items() , key = operator.itemgetter(1) , reverse=True )

		Keys = []
		for t in sorted_x[:threshold]:
			Keys.append( t[0] )
		return Keys

	def normalize_edges( self , nxGraph ):

		links = []
		max_w = -1
		for e in nxGraph.edges_iter():
			weight = nxGraph[e[0]][e[1]]["weight"]
			if weight>max_w: 
				max_w=weight

		for e in nxGraph.edges_iter():
			s = e[0]
			t = e[1]
			nxGraph[s][t]["weight"] = (nxGraph[s][t]["weight"]/max_w)

		return nxGraph

	def BiGraph(self , Args , nodeTypes , nodeDicts ):

		nodesA_toB = nodeDicts[0]
		nodesB_toA = nodeDicts[1]

		nodesA = Args["nodesA"]
		filter_method = Args["filternodesby"]
		max_opposneigh = Args["max_oppos"]
		weight_normalization = Args["normalization"]

		# print("")
		# print( "= = = = = = = = = = = = = = ")
		# print( "nodesA:",nodesA)
		# print( "filter by:",filter_method)
		# print( "max voisinage:",max_opposneigh)
		# print( "normalize?:",weight_normalization)
		# print( "= = = = = = = = = = = = = = ")
		# print("")

		Nodes_str2int = {}
		Nodes_int2str = {}
		it = 1
		
		# B -> B depuis AxB*
		Graph = Utils()
		nb_A = len( nodesA_toB.keys() )
		for ID in nodesA_toB:
			Nodes_int2str[it] = ID
			Nodes_str2int[ID] = it
			it +=1
			if len(nodesA_toB[ID].keys())>0:
				# Keys = getattr(self, filter_method)( nodesA_toB[ID] , nodesB_toA , nb_A ,  max_opposneigh )
				Keys = list(nodesA_toB[ID].keys())
				Graph.addCompleteSubGraph ( Keys )
		GraphB = Graph.G
		GraphB.remove_nodes_from(nx.isolates(GraphB))
		GraphB = self.normalize_edges( GraphB )
		print (len(GraphB))


		# A -> A depuis BxA*
		Graph = Utils()
		nb_B = len( nodesB_toA.keys() )
		for ID in nodesB_toA:
			Nodes_int2str[it] = ID
			Nodes_str2int[ID] = it
			it +=1
			if len(nodesB_toA[ID].keys())>0:
				# Keys = getattr(self, filter_method)( nodesB_toA[ID] , nodesA_toB , nb_B ,  max_opposneigh )
				Keys = list(nodesB_toA[ID].keys())
				Graph.addCompleteSubGraph ( Keys )
		GraphA = Graph.G
		GraphA.remove_nodes_from(nx.isolates(GraphA))
		GraphA = self.normalize_edges( GraphA )
		print (len(GraphA))


		Nodes = []
		Links = []

		# JSON B -> B
		Nodes,Links = self.SameKindNodes(  Nodes_str2int , GraphB , nodeTypes[1] , False , Nodes , Links , False)

		# JSON A -> A
		Nodes,Links = self.SameKindNodes(  Nodes_str2int , GraphA , nodeTypes[0] , "square" , Nodes , Links , False)


		# JSON A -> B
		c = len( Links )+1
		for s in GraphA.nodes_iter():
			# print s , "->"
			# print "\t" , nodesA_toB[s].keys()
			for t in nodesA_toB[s]:
				if GraphB.nodes(Nodes_str2int[t]):
					link = {
						"id": str(c),
						"s":Nodes_str2int[s],
						"t":Nodes_str2int[t],
						"w": 1
					}
					Links.append(link)
					c += 1

		graphArray = {
			"nodes":Nodes,
			"links":Links,
		}

		# pprint.pprint(graphArray["clusters"])


		print ("nodesA:",len(nodesA_toB.keys()))
		print ("nodesB:",len(nodesB_toA.keys()))
		print ("fini")
		# results = [len(nodesA_toB.keys()) , len(nodesB_toA.keys()) , len(Links)]

		return [graphArray , nodesA_toB.keys() , nodesB_toA.keys() , len(Links)]

	def BiGraph_2(self , nodesA_toB , nodesB_toA ):

		it = 1
		
		# B -> B depuis AxB*
		Graph = Utils()
		nb_A = len( nodesA_toB.keys() )
		for ID in nodesA_toB:
			if len(nodesA_toB[ID].keys())>0:
				# Keys = getattr(self, filter_method)( nodesA_toB[ID] , nodesB_toA , nb_A ,  max_opposneigh )
				Keys = list(nodesA_toB[ID].keys())
				Graph.addCompleteSubGraph ( Keys )
		GraphB = Graph.G
		GraphB.remove_nodes_from(nx.isolates(GraphB))
		GraphB = self.normalize_edges( GraphB )
		graphArray = {
			"nodes":[],
			"links":[],
			"G": GraphB
		}
		return graphArray
		# print (len(GraphB))

		Nodes = []
		Links = []

		# JSON B -> B
		Nodes,Links = self.SameKindNodes(  None , GraphB , "mesh_term" , False , Nodes , Links , False)

		# # JSON A -> B
		# c = len( Links )+1
		# for cID in nodesA_toB:
		# 	# print( cID , "->")
		# 	# print( "\t" , nodesA_toB[cID].keys())
		# 	for ngram in nodesA_toB[cID]:
		# 		if GraphB.nodes( ngram ):
		# 			link = {
		# 				"id": str(c),
		# 				"s":cID,
		# 				"t":ngram,
		# 				"w": 1
		# 			}
		# 			Links.append(link)
		# 			c += 1


		# pprint.pprint(graphArray["clusters"])


		print ("nodesA:",len(nodesA_toB.keys()))
		print ("nodesB:",len(nodesB_toA.keys()))
		# print ("fini")
		# results = [len(nodesA_toB.keys()) , len(nodesB_toA.keys()) , len(Links)]

		return [graphArray , nodesA_toB.keys() , nodesB_toA.keys() , len(Links)]

# if __name__ == "__main__":
# 	main()
# 	# from CNRS.Main import CNRSQuerier